#!/bin/sh
#
# Based on esp-open-sdk
#

SDK_VERSION=2.1.0
SDK_URL="https://github.com/espressif/ESP8266_NONOS_SDK/archive/v2.1.0.tar.gz"
SDK_PATCHES=""
DOC_DIR=/usr/share/doc/packages/fetch-esp8266-sdk-2.1.0/

if [ -z $TOOLCHAIN ]; then
  TOOLCHAIN=/opt/cross/xtensa-lx106-g++/
fi
if [ -z $SDK_DIR ]; then
  SDK_DIR="ESP8266_NONOS_SDK-$SDK_VERSION"
fi

wget -O - http://bbs.espressif.com > /dev/null 2> /dev/null
if [ $? != 0 ]; then
 echo "Cannot connect to webserver to download SDK. Abort"
 exit 0
fi

if [ "`id -u`" != "0" ]; then
 echo "error: You must be root to use this program!"
 exit 1
fi

if [ -f /etc/sysconfig/proxy ] ; then
   . /etc/sysconfig/proxy
fi

if test "x$PROXY_ENABLED" != "xno"; then
  if test -n "$HTTP_PROXY" ; then
    export http_proxy="$HTTP_PROXY"
  fi
fi

if [ -z $http_proxy ]; then
  echo 
  echo "note: No proxy is used. Please set the environment variable \"http_proxy\""
  echo "note: to your favorite proxy, if you want to use a proxy for the download."
  echo "note:"
  echo "note:   bash: export http_proxy=\"http://proxy.example.com:3128/\""
  echo "note:   tcsh: setenv http_proxy \"http://proxy.example.com:3128/\""
fi

OLDPWD=$(pwd)
DIR=$(mktemp -d)
if [ $? != 0 ]; then
  echo "Could not create temporary directory! (mktemp failed)"
  exit 1
fi
cd $DIR

echo "Download SDK"
wget -O sdk.tar.gz ${SDK_URL}
tar xzf sdk.tar.gz

if [ "x$SDK_PATCHES" != "x" ]; then
  echo "Download additional libraries"
  for PATCH in $SDK_PATCHES; do 
    wget -O patch.zip $PATCH
    unzip -q -o patch.zip
    rm patch.zip
  done
  mv *.a ${SDK_DIR}/lib/
fi

echo "Patch c_types.h of the SDK"
patch -N -f -d ${SDK_DIR} -p1 < ${OLDPWD}/c_types-c99.patch

echo "Creating irom version of libc..."
${TOOLCHAIN}/bin/xtensa-lx106-elf-objcopy --rename-section .text=.irom0.text --rename-section .literal=.irom0.literal ${TOOLCHAIN}/xtensa-lx106-elf/sysroot/lib/libc.a ${TOOLCHAIN}/xtensa-lx106-elf/sysroot/lib/libcirom.a;

echo "File list for packaging:"
DIR=${SDK_DIR}/include/
find $DIR -name *.h -exec echo {} \; | sed -e "s#$DIR#%ghost %{TOOLCHAIN}/xtensa-lx106-elf/sysroot/usr/include/#g"
DIR=${SDK_DIR}/lib/
find $DIR -name *.a -exec echo {} \; | sed -e "s#$DIR#%ghost %{TOOLCHAIN}/xtensa-lx106-elf/sysroot/usr/lib/#g"
if [ ! -z DOC_DIR ]; then
  DIR=${SDK_DIR}/documents/
  find $DIR -name *.pdf -exec echo '{}"' \; | sed -e "s#$DIR#%ghost \"%{_docdir}/%{name}-%{version}/#g"
fi

echo "Installing vendor SDK headers into toolchain sysroot"
cp -Rf ${SDK_DIR}/include/* ${TOOLCHAIN}/xtensa-lx106-elf/sysroot/usr/include/
echo "Installing vendor SDK libs into toolchain sysroot"
cp -Rf ${SDK_DIR}/lib/* ${TOOLCHAIN}/xtensa-lx106-elf/sysroot/usr/lib/
echo "Installing vendor SDK bins into toolchain sysroot"
cp -Rf ${SDK_DIR}/bin/* ${TOOLCHAIN}/bin/
echo "Installing vendor SDK linker scripts into toolchain sysroot"
sed -e 's/\r//' ${SDK_DIR}/ld/eagle.app.v6.ld | sed -e s@../ld/@@ >${TOOLCHAIN}/xtensa-lx106-elf/sysroot/usr/lib/eagle.app.v6.ld
sed -e 's/\r//' ${SDK_DIR}/ld/eagle.rom.addr.v6.ld >${TOOLCHAIN}/xtensa-lx106-elf/sysroot/usr/lib/eagle.rom.addr.v6.ld
mkdir -p ${TOOLCHAIN}/ld
cp ${TOOLCHAIN}/xtensa-lx106-elf/sysroot/usr/lib/eagle.app.v6.ld ${TOOLCHAIN}/ld/
cp ${TOOLCHAIN}/xtensa-lx106-elf/sysroot/usr/lib/eagle.rom.addr.v6.ld ${TOOLCHAIN}/ld/

rm -f ${TOOLCHAIN}/xtensa-lx106-elf/sysroot/usr/include/c_types.h.orig

echo "Installing documentation"
if [ ! -z DOC_DIR ]; then
  mv License ${DOC_DIR}/LICENSE
  cp -Rf ${SDK_DIR}/documents/* ${DOC_DIR}
fi

cd ${OLDPWD}
rm -rf ${DIR}

echo "ESP8266 SDK installed."
echo
echo "Use the following setting in your Makefile:"
echo "XTENSA_TOOLS_ROOT = ${TOOLCHAIN}/bin"
echo "SDK_BASE          = ${TOOLCHAIN}"
echo "ESPTOOL           = esptool.py"
